@@ -58,7 +58,7 @@ sub defpkg {
 sub defclass {
 }
 
-our $VERSION = '0.24'; # VERSION
+our $VERSION = '0.25'; # VERSION
 
 1;
 # ABSTRACT: Some shortcuts
@@ -73,7 +73,7 @@ Perinci::Easy - Some shortcuts
 
 =head1 VERSION
 
-version 0.24
+version 0.25
 
 =head1 DESCRIPTION
 
@@ -83,6 +83,44 @@ This module provides some shortcuts.
 
 L<Perinci>
 
+=head1 DESCRIPTION
+
+
+This module has L<Rinci> metadata.
+
+=head1 FUNCTIONS
+
+
+None are exported by default, but they are exportable.
+
+=head2 defsub() -> [status, msg, result, meta]
+
+Define a subroutine.
+
+This is just a shortcut to define subroutine and meta together so instead of:
+
+    our %SPEC;
+    $SPEC{foo} = {
+        v => 1.1,
+        summary => 'Blah ...',
+    };
+    sub foo {
+        ...
+    }
+
+you write:
+
+    defsub name=>'foo', summary=>'Blah ...',
+        code=>sub {
+            ...
+        };
+
+No arguments.
+
+Return value:
+
+Returns an enveloped result (an array). First element (status) is an integer containing HTTP status code (200 means OK, 4xx caller error, 5xx function error). Second element (msg) is a string containing error message, or 'OK' if status is 200. Third element (result) is optional, the actual result. Fourth element (meta) is called result metadata and is optional, a hash that contains extra information.
+
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
